@@ -20,6 +20,9 @@ class Spline:
                 return self.__eval(val, coeffs)
         return None
 
+    def __repr__(self):
+        return f"Spline object over domain(s): {self.domains_lists} with coefficients: {self.coeff_lists}"
+
     def get_range(self):
         return np.min(self.domains_lists), np.max(self.domains_lists)
 
@@ -62,7 +65,7 @@ class AdjusterGui:
         self.collision_points = copy.deepcopy(self.points)
         self.collision_points[1][1]["pos"] += 1
 
-        self.generator_func = gen_spline_5
+        self.generator_func = gen_spline  # gen_poly_5
 
         plt.ion()
         self.fig, self.axs = plt.subplots(1, 3)
@@ -296,7 +299,7 @@ class AdjusterGui:
             return self.p3_acc_y_scale.get()
 
     def update_order(self):
-        self.generator_func = gen_spline_5 if self.order_bool.get() else gen_spline_3
+        self.generator_func = gen_poly_5 if self.order_bool.get() else gen_spline  # gen_poly_3
         # print(f"Update order: {self.order_bool.get()}")
         self.update_points(None)
 
@@ -330,12 +333,14 @@ class AdjusterGui:
         plot_lst = []
 
         plot_points_normal = generate_plot_data(self.points,
-                                                gen_splines_xy(self.points, spline_generator=self.generator_func),
+                                                gen_xy(self.points, generator=self.generator_func),
                                                 step=self.plot_step_size)
         plot_lst.append(plot_points_normal)
         point_lst.append(self.points)
         if self.show_collision:
-            plot_points_collision = generate_plot_data(self.points, gen_splines_xy(self.collision_points, spline_generator=self.generator_func), step=self.plot_step_size)
+            plot_points_collision = generate_plot_data(self.points, gen_xy(self.collision_points,
+                                                                           generator=self.generator_func),
+                                                       step=self.plot_step_size)
             plot_lst.append(plot_points_collision)
             point_lst.append(self.collision_points)
 
@@ -346,15 +351,105 @@ class AdjusterGui:
 # =================== END CLASSES     ===============================================
 # =================== START FUNCTIONS ===============================================
 
-def gen_spline_5(point_params: list, verbose=False) -> list:
-    if len(point_params) < 2:
-        raise AttributeError("Not enough points provided")
+def gen_spline(point_params: list, verbose=False) -> list:
+    if len(point_params) != 3:
+        raise AttributeError("Not provided 3 point.")
     if verbose:
         print(f"Generating splines for points {point_params}")
 
+    t = [point["t"] for point in point_params]
+    pos = [point["pos"] for point in point_params]
+    vel = [point_params[0]["vel"], point_params[2]["vel"]]
+    acc = [point_params[0]["acc"], point_params[2]["acc"]]
+    # print(t, pos, vel, acc)
+
+    A = np.array([
+        [t[0] ** 4, t[0] ** 3, t[0] ** 2, t[0], 1, 0, 0, 0, 0, 0],
+        [4 * t[0] ** 3, 3 * t[0] ** 2, 2 * t[0], 1, 0, 0, 0, 0, 0, 0],
+        [12 * t[0] ** 2, 6 * t[0], 2, 0, 0, 0, 0, 0, 0, 0],
+        [t[1] ** 4, t[1] ** 3, t[1] ** 2, t[1], 1, 0, 0, 0, 0, 0],
+        [0, 0, 0, 0, 0, t[1] ** 4, t[1] ** 3, t[1] ** 2, t[1], 1],
+        [0, 0, 0, 0, 0, t[2] ** 4, t[2] ** 3, t[2] ** 2, t[2], 1],
+        [0, 0, 0, 0, 0, 4 * t[2] ** 3, 3 * t[2] ** 2, 2 * t[2], 1, 0],
+        [0, 0, 0, 0, 0, 12 * t[2] ** 2, 6 * t[2], 2, 0, 0],
+        [4 * t[1] ** 3, 3 * t[1] ** 2, 2 * t[1], 1, 0, -4 * t[1] ** 3, -3 * t[1] ** 2, -2 * t[1], -1, 0],
+        [12 * t[1] ** 2, 6 * t[1], 2, 0, 0, -12 * t[1] ** 2, -6 * t[1], -2, 0, 0]])
+
+    b = np.array([pos[0], vel[0], acc[0], pos[1], pos[1], pos[2], vel[1], acc[1], 0, 0])
+    coeffs = np.linalg.solve(A, b)
+
+    spline_coefficients = [coeffs[:5], coeffs[5:]]
+    spline_domains = [t[:2], t[1:]]
+    sp = Spline(spline_coefficients, spline_domains)
+    if verbose:
+        print(f"Solving the System A*x=b : \nA: {A}\nb: {b}")
+        print(f"x: {coeffs}")
+        print(sp, "\n")
+
+    return sp
+
+
+def gen_poly_auto(point_params: list, verbose=False) -> list:
+    if len(point_params) < 2:
+        raise AttributeError("Not enough points provided")
+    if verbose:
+        print(f"Generating polynomials for points {point_params}")
+
     # tstart = time.perf_counter()
-    spline_coefficients = []
-    spline_domains = []
+    poly_coefficients = []
+    poly_domains = []
+    for i, _ in enumerate(point_params[:-1]):
+        t = [point_params[i + j]["t"] for j in (0, 1)]
+
+        pos = [point_params[i + j]["pos"] if "pos" in point_params[i + j] else None for j in (0, 1)]
+        vel = [point_params[i + j]["vel"] if "vel" in point_params[i + j] else None for j in (0, 1)]
+        acc = [point_params[i + j]["acc"] if "acc" in point_params[i + j] else None for j in (0, 1)]
+
+        # print(f"Using values: pos: {pos}, vel: {vel}, acc: {acc}")
+
+        A = np.array([
+            [t[0] ** 5, t[0] ** 4, t[0] ** 3, t[0] ** 2, t[0], 1],
+            [t[1] ** 5, t[1] ** 4, t[1] ** 3, t[1] ** 2, t[1], 1],
+            [5 * t[0] ** 4, 4 * t[0] ** 3, 3 * t[0] ** 2, 2 * t[0], 1, 0],
+            [5 * t[1] ** 4, 4 * t[1] ** 3, 3 * t[1] ** 2, 2 * t[1], 1, 0],
+            [20 * t[0] ** 3, 12 * t[0] ** 2, 6 * t[0], 2, 0, 0],
+            [20 * t[1] ** 3, 12 * t[1] ** 2, 6 * t[1], 2, 0, 0]])
+
+        b = np.array([pos[0], pos[1], vel[0], vel[1], acc[0], acc[1]], dtype=float)
+        remove_ctr = 0
+
+        for idx, val in enumerate(b):
+
+            if np.isnan(val):
+                # print(A, b)
+                A = np.delete(A, 0, 1)  # erste Spalte von A entfernen
+                A = np.delete(A, idx - remove_ctr, 0)  # Zeile aus A entfernen
+                b = np.delete(b, idx - remove_ctr, 0)  # Element aus b entfernen
+                remove_ctr += 1
+                if verbose:
+                    print(f"Missing entry, reducing polynomial degree")
+                # print(A, b)
+
+        coeffs = np.linalg.solve(A, b)
+        if verbose:
+            print(f"Solving the following System A*x=b : \nA: {A}\nb: {b}")
+            print(f"Coefficients: {coeffs}")
+        poly_coefficients.append(coeffs)
+        poly_domains.append(t)
+    # tend = time.perf_counter()
+    # print(f"Time passed while calculating splines for one dimension: {tend-tstart}")
+    return Spline(poly_coefficients, poly_domains)
+
+
+def gen_poly_5(point_params: list, verbose=False) -> list:
+    if len(point_params) < 2:
+        raise AttributeError("Not enough points provided")
+    if verbose:
+        print(f"Generating polynomials for points {point_params}")
+
+    # tstart = time.perf_counter()
+    poly_coefficients = []
+    poly_domains = []
     for i, _ in enumerate(point_params[:-1]):
         t = [point_params[i + j]["t"] for j in (0, 1)]
 
@@ -380,22 +475,22 @@ def gen_spline_5(point_params: list, verbose=False) -> list:
         # spline = lambda t, koeffs=koeffs: koeffs[0] * t ** 5 + koeffs[1] * t ** 4 + koeffs[2] * t ** 3 + koeffs[
         #     3] * t ** 2 + koeffs[
         #                                       4] * t + koeffs[5]
-        spline_coefficients.append(coeffs)
-        spline_domains.append(t)
+        poly_coefficients.append(coeffs)
+        poly_domains.append(t)
     # tend = time.perf_counter()
     # print(f"Time passed while calculating splines for one dimension: {tend-tstart}")
-    return Spline(spline_coefficients, spline_domains)
+    return Spline(poly_coefficients, poly_domains)
 
 
-def gen_spline_3(point_params: list, verbose=False) -> Spline:
+def gen_poly_3(point_params: list, verbose=False) -> Spline:
     if len(point_params) < 2:
         raise AttributeError("Not enough points provided")
     if verbose:
-        print(f"Generating spline_coefficients for points {point_params}")
+        print(f"Generating polynomials for points {point_params}")
 
     # tstart = time.perf_counter()
-    spline_coefficients = []
-    spline_domains = []
+    poly_coefficients = []
+    poly_domains = []
     for i, _ in enumerate(point_params[:-1]):
         t = [point_params[i + j]["t"] for j in (0, 1)]
 
@@ -417,22 +512,30 @@ def gen_spline_3(point_params: list, verbose=False) -> Spline:
             print(f"Coefficients: {coeffs}")
         # spline = lambda t, koeffs=koeffs: koeffs[0] * t ** 3 + koeffs[1] * t ** 2 + koeffs[2] * t + koeffs[3]
         # spline_coefficients.append(spline)
-        spline_coefficients.append(coeffs)
-        spline_domains.append(t)
+        poly_coefficients.append(coeffs)
+        poly_domains.append(t)
 
     # tend = time.perf_counter()
     # print(f"Time passed while calculating spline_coefficients for one dimension: {tend-tstart}")
-    return Spline(spline_coefficients, spline_domains)
+    return Spline(poly_coefficients, poly_domains)
 
 
-def gen_splines_xy(point_param_lists: list, spline_generator=gen_spline_3) -> list:
+def gen_xy(point_param_lists: list, generator=gen_poly_3) -> list:
     # tstart = time.perf_counter_ns()
     points_x = [p[0] for p in point_param_lists]
     points_y = [p[1] for p in point_param_lists]
     # tend = time.perf_counter_ns()
     # print(f"Time Passed for calculationd 2D splines: {tend-tstart}ns")
 
-    return [spline_generator(points_x), spline_generator(points_y)]
+    return [generator(points_x, verbose=False), generator(points_y, verbose=False)]
+
+
+def gen_xyz(point_param_lists: list, generator=gen_poly_3) -> list:
+    points_x = [p[0] for p in point_param_lists]
+    points_y = [p[1] for p in point_param_lists]
+    points_z = [p[2] for p in point_param_lists]
+
+    return [generator(points_x), generator(points_y), poly(points_z)]
 
 
 def setup_plot(axs):
@@ -455,15 +558,15 @@ def setup_plot(axs):
     axs[2].set_title("Combined Splines in xy-plane")
 
 
-def generate_plot_data(point_param_lists: list, splines_xy_list=None, step=0.01):
-    splines_xy = splines_xy_list
-    if splines_xy_list is None:
-        splines_xy = gen_splines_xy(point_param_lists)
+def generate_plot_data(point_param_lists: list, poly_xy_list=None, step=0.01):
+    poly_xy = poly_xy_list
+    if poly_xy_list is None:
+        poly_xy = gen_xy(point_param_lists)
 
     t = [np.arange(point_param_lists[i][0]["t"], point_param_lists[i + 1][0]["t"] + step, step) for i in
          range(len(point_param_lists) - 1)]
-    x_pts = [list(map(splines_xy[0], t_)) for t_ in t]
-    y_pts = [list(map(splines_xy[1], t_)) for t_ in t]
+    x_pts = [list(map(poly_xy[0], t_)) for t_ in t]
+    y_pts = [list(map(poly_xy[1], t_)) for t_ in t]
 
     return t, x_pts, y_pts
 
@@ -516,8 +619,8 @@ def plot_from_plot_data(plot_data: list, point_parameter_lists: list, pyplot_axs
             axs[2].plot(point[0]["pos"], point[1]["pos"], "xk")
 
 
-def plot_splines(point_param_lists: list, splines_xy_list=None, step=0.01, pyplot_axs=None, update=False):
-    plot_from_plot_data([generate_plot_data(point_param_lists, splines_xy_list, step)], [point_param_lists], pyplot_axs,
+def plot(point_param_lists: list, poly_xy_list=None, step=0.01, pyplot_axs=None, update=False):
+    plot_from_plot_data([generate_plot_data(point_param_lists, poly_xy_list, step)], [point_param_lists], pyplot_axs,
                         update)
 
 
@@ -527,6 +630,15 @@ def get_example_point_data() -> list:
             [{"t": 1.0, "pos": 1, "vel": -5, "acc": -0.25}, {"t": 1, "pos": 0, "vel": -0.25, "acc": 0.1}]]
 
 
+def get_example_point_data_3D() -> list:
+    return [[{"t": 0, "pos": -1, "vel": -5, "acc": 10}, {"t": 0, "pos": 0, "vel": 0.1, "acc": 0.1},
+             {"t": 0, "pos": 0, "vel": -0.01, "acc": -0.01}],
+            [{"t": 0.5, "pos": 0, "vel": 20, "acc": 0}, {"t": 0.5, "pos": 1, "vel": 0, "acc": -2},
+             {"t": 0.5, "pos": 0.1, "vel": 0, "acc": 0.1}],
+            [{"t": 1.0, "pos": 1, "vel": -5, "acc": -0.25}, {"t": 1, "pos": 0, "vel": -0.25, "acc": 0.1},
+             {"t": 1, "pos": 0, "vel": 0.01, "acc": 0.01}]]
+
+
 if __name__ == "__main__":
     # ex_points = get_example_point_data()
     # fig, axs = plt.subplots(1, 3)
@@ -535,5 +647,19 @@ if __name__ == "__main__":
     # plt.show()
     win_tk = AdjusterGui()
 
+    # data = get_example_point_data_3D()
+    # del data[0][0]["vel"]
+    # del data[1][0]["vel"]
+    # print(data)
+    # points_x = [p[0] for p in data]
+    # points_y = [p[1] for p in data]
+    # points_z = [p[2] for p in data]
+    # gen_spline_auto(points_x)
+
+    # splines_3D = gen_splines_xyz(get_example_point_data_3D(), gen_spline_5)
+    # print(splines_3D[0])
+    # print(splines_3D[1])
+    # print(splines_3D[2])
+
 # TODO
 # Make scaling on axes the same
